package net.wanji.common.utils.tool;

import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.time.temporal.TemporalUnit;
import java.util.Date;
import java.util.TimeZone;

/**
 * 时间工具类
 *
 * @author zhouleilei
 * @version 1.0
 * @date 2024/11/3 13:16
 */
public class LocalDateTimeUtil {
    public static final String TIMEFORMATTER = "yyyy-MM-dd HH:mm:ss";
    public static final String HOURFORMATTER = "HH";
    public static final String DAYFORMATTER = "yyyy-MM-dd";
    public static final String MONTHFORMATTER = "yyyy-MM";
    public static final String YEARFORMATTER = "yyyy";
    public static final String SIMPLE = "yyyyMMddHHmmss";
    public static final String TIMEFORMATTERTILT = "yyyy/MM/dd HH:mm:ss";

    private LocalDateTimeUtil() {

    }

    //Date转换为LocalDateTime
    public static LocalDateTime convertDateToLDT(Date date) {
        return LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault());
    }

    //LocalDateTime转换为Date
    public static Date convertLDTToDate(LocalDateTime time) {
        return Date.from(time.atZone(ZoneId.systemDefault()).toInstant());
    }

    public static java.sql.Date localTimeToDate(LocalDateTime localDateTime) {
        return new java.sql.Date(localDateTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli());
    }

    //获取指定日期的毫秒
    public static Long getMilliByTime(LocalDateTime time) {
        return time.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
    }

    //获取指定日期的秒
    public static Long getSecondsByTime(LocalDateTime time) {
        return time.atZone(ZoneId.systemDefault()).toInstant().getEpochSecond();
    }

    //获取指定时间的指定格式
    public static String formatTime(LocalDateTime time, String pattern) {
        return time.format(DateTimeFormatter.ofPattern(pattern));
    }

    //获取指定时间的指定格式
    public static String formatTimeStamp(Long timestamp, String pattern) {
        return formatTime(parseLongToDateTime(timestamp), pattern);
    }

    //获取当前时间的指定格式
    public static String formatNow(String pattern) {
        return formatTime(LocalDateTime.now(), pattern);
    }

    //日期加上一个数,根据field不同加不同值,field为ChronoUnit.*
    public static LocalDateTime plus(LocalDateTime time, long number, TemporalUnit field) {
        return time.plus(number, field);
    }

    //日期减去一个数,根据field不同减不同值,field参数为ChronoUnit.*
    public static LocalDateTime minu(LocalDateTime time, long number, TemporalUnit field) {
        return time.minus(number, field);
    }

    //获取当前时间前N个月份
    public static String getLastNMonth(int n) {
        LocalDateTime lastNMonth = LocalDateTime.now().minusMonths(n);
        DateTimeFormatter formatters = DateTimeFormatter.ofPattern(MONTHFORMATTER);
        return formatters.format(lastNMonth);
    }

    /**
     * 获取两个日期的差  field参数为ChronoUnit.*
     *
     * @param startTime
     * @param endTime
     * @param field     单位(年月日时分秒)
     * @return
     */
    public static long betweenTwoTime(LocalDateTime startTime, LocalDateTime endTime, ChronoUnit field) {
        Period period = Period.between(LocalDate.from(startTime), LocalDate.from(endTime));
        if (field == ChronoUnit.YEARS) {
            return period.getYears();
        }
        if (field == ChronoUnit.MONTHS) {
            return (long) (period.getYears() * 12 + period.getMonths());
        }
        return field.between(startTime, endTime);
    }

    //获取一天的开始时间，2017,7,22 00:00
    public static LocalDateTime getDayStart(LocalDateTime time) {
        return time.withHour(0)
                .withMinute(0)
                .withSecond(0);
        //.withNano(0);
    }

    //获取一天的结束时间，2017,7,22 23:59:59.999999999
    public static LocalDateTime getDayEnd(LocalDateTime time) {
        return time.withHour(23)
                .withMinute(59)
                .withSecond(59);
        //.withNano(999999999);
    }


    /**
     * 把长整数转化为LocalDateTime
     *
     * @param timestamp 时间长整数
     * @return
     */
    public static LocalDateTime parseLongToDateTime(Long timestamp) {
        return LocalDateTime.ofInstant(Instant.ofEpochMilli(timestamp), TimeZone.getDefault().toZoneId());
    }

    /**
     * 把时间戳转化为LocalDateTime
     *
     * @param timestamp 时间戳
     * @return
     */
    public static LocalDateTime getDateTimeOfTimestamp(long timestamp) {
        Instant instant = Instant.ofEpochMilli(timestamp);
        ZoneId zone = ZoneId.systemDefault();
        return LocalDateTime.ofInstant(instant, zone);
    }

    /**
     * 将日期字符串转化为LocalDateTime
     *
     * @param time   时间字符串
     * @param format 格式
     * @return
     */
    public static LocalDateTime parseStringToDateTime(String time, String format) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern(format);
        return LocalDateTime.parse(time, df);
    }

    /**
     * 将日期字符串转化为LocalDateTime
     *
     * @param time   时间字符串
     * @param format 格式
     * @return
     */
    public static LocalDateTime parseStringToDateTimeTwo(String time, String format) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern(format);
        return LocalDateTime.parse(checkFormat(time), df);
    }

    public static void main(String[] args) {
        System.out.println("--"+new Date());
        String date = "2021/10/22 17:45:20";
        String nowStageStartTime = date == null ? "" : date.toString().replaceAll("\"", "");
        LocalDateTime localDateTime = parseStringToDateTimeTwo(nowStageStartTime, LocalDateTimeUtil.TIMEFORMATTERTILT);
        String time = nowStageStartTime.isEmpty() ? "0" : Long.toString(LocalDateTimeUtil.betweenTwoTime(localDateTime, LocalDateTime.now(), ChronoUnit.SECONDS));
        System.out.println(time);
    }

    /**
     * 将日期字符串转化为指定格式的日期字符串
     *
     * @param time 时间字符串
     * @return
     */
    public static String parseStringToFormatString(String time, String oldFormat, String newFormat) {
        if (time == null || time.isEmpty()) return time;
        DateTimeFormatter df = DateTimeFormatter.ofPattern(oldFormat);
        return LocalDateTime.parse(checkFormat(time), df).format(DateTimeFormatter.ofPattern(newFormat));
    }

    /**
     * 时间格式规范,仅标准格式
     */
    public static String checkFormat(String time) {
        String timeNew = time.substring(0, time.indexOf(" "));
        String spit = timeNew.substring(4, 5);
        String year = timeNew.substring(0, timeNew.indexOf(spit));
        String month = timeNew.substring(timeNew.indexOf(spit) + 1, timeNew.lastIndexOf(spit));
        String day = timeNew.substring(timeNew.lastIndexOf(spit) + 1);
        //弄时分秒
        String shifenmiao = time.substring(time.indexOf(" ") + 1);
        String hour = shifenmiao.substring(0, shifenmiao.indexOf(":"));
        String min = shifenmiao.substring(shifenmiao.indexOf(":") + 1, shifenmiao.lastIndexOf(":"));
        String sec = shifenmiao.substring(shifenmiao.lastIndexOf(":") + 1);
        return MessageFormat.format("{0}{1}{2}{3}{4}{5}{6}{7}{8}{9}{10}",
                year, spit, String.format("%02d", Integer.valueOf(month)), spit, String.format("%02d", Integer.valueOf(day)),
                " ",
                String.format("%02d", Integer.valueOf(hour)), ":", String.format("%02d", Integer.valueOf(min)), ":", String.format("%02d", Integer.valueOf(sec)));
    }


    /**
     * 获取当前时间的前一天时间值 如：2019-10-12 12:10:08 --> 2019-10-11 12:10:08
     *
     * @param time 时间
     */
    public static String getYesterdayString(String time) {
        return LocalDateTimeUtil.parseStringToDateTime(time, TIMEFORMATTER).plusDays(-1).format(DateTimeFormatter.ofPattern(TIMEFORMATTER));
    }

    /**
     * 获取指定时间所在月的第一天
     *
     * @param dateTime
     * @return
     */
    public static LocalDateTime getMonthFirstDay(LocalDateTime dateTime) {
        LocalDateTime firstday = dateTime.with(TemporalAdjusters.firstDayOfMonth());
        firstday = firstday.withHour(0).withMinute(0).withSecond(0);
        return firstday;
    }

    /**
     * 获取指定时间所在月的最后一天
     *
     * @param dateTime
     * @return
     */
    public static LocalDateTime getMonthLastDay(LocalDateTime dateTime) {
        LocalDateTime lastDay = dateTime.with(TemporalAdjusters.lastDayOfMonth());
        lastDay = lastDay.withHour(23).withMinute(59).withSecond(59);
        return lastDay;
    }

    /**
     * 得到现在分钟（Integer）
     *
     * @author lzs
     */
    public static Integer getNowMinuteInt() {
        Date currentTime = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String dateString = formatter.format(currentTime);
        String min;
        min = dateString.substring(14, 16);
        return Integer.valueOf(min);
    }

    /**
     * 得到现在小时(Integer)
     *
     * @author lzs
     */
    public static Integer getNowHourInt() {
        Date currentTime = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String dateString = formatter.format(currentTime);
        String hour;
        hour = dateString.substring(11, 13);
        return Integer.valueOf(hour);
    }

    /**
     * 获得日期是周几
     *
     * @param localDateTime
     * @return
     */
    public static int getDayOfWeekNum(LocalDateTime localDateTime) {
        return localDateTime.getDayOfWeek().getValue();
    }
}
